@@ -1,3 +1,6 @@
+// Derived from libstdc++/12048 by LJR <ljrittle@acm.org> with
+// reminder from Petur Runolfsson <peturr02@ru.is>.
+
 // Copyright (C) 2003-2016 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -16,8 +19,8 @@
 // <http://www.gnu.org/licenses/>.
 
 #include <iostream>
-#include <cstdio>
 #include <cwchar>
+#include <cstdio>
 #include <testsuite_hooks.h>
 
 void
@@ -25,13 +28,12 @@ test01()
 {
   VERIFY( std::freopen("cin_unget-1.txt", "r", stdin) );
 
-  wchar_t c1;
-  std::wint_t c2;
-  std::wcin.get(c1);
-  std::wcin.unget();
-  VERIFY( std::wcin.good() );
-  c2 = std::fgetwc(stdin);
-  VERIFY( c2 == std::char_traits<wchar_t>::to_int_type(c1) );
+  wchar_t buf[2];
+  VERIFY( std::wcin.rdbuf()->sgetn(buf, 2) == 2 );
+  std::wint_t c1 = std::wcin.rdbuf()->sungetc();
+  std::wint_t c2 = std::wcin.rdbuf()->sbumpc();
+  VERIFY( c1 == std::char_traits<wchar_t>::to_int_type(buf[1]) );
+  VERIFY( c2 == c1 );
 }
 
 int main(void)
